<?php

class MeowPro_MWL_Core {
  private $core = null;
  private $item = 'Meow Lightbox Pro';
  private $isEnqueued = false;

  public function __construct( $core ) {
    $this->core = $core;

    // Additional functions for Pro
    add_action( 'init', array( $this, 'init' ) );

    // The Lightbox should be completely off if the request is asynchronous
    if ( MeowCommon_Helpers::is_asynchronous_request() || MeowCommon_Helpers::is_divi_builder() ) {
			return;
    }

    // MODE: Output Buffering
    if ( $this->core->isObMode ) {
      // This doesn't work well with the conditional loading of the scripts, so we need to load it anyway.
      add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
    }
    // MODE: Standard (Responsive Images)
    else {
      // This doesn't work well with the conditional loading of the scripts, so we need to load it
      add_action( 'mwl_lightbox_added', array( $this, 'lightbox_added' ), 10, 1 );
    }
  }

  function init() {
    // Common behaviors, license, update system, etc.
    new MeowCommonPro_Licenser( MWL_PREFIX, MWL_ENTRY, MWL_DOMAIN, $this->item, MWL_VERSION );
  }

  public function lightbox_added( $mediaId ) {
		//error_log("DEBUG: LIGHTBOX_ADDED " . $mediaId );
		if ( $this->isEnqueued ) {
			return;
		}
		$this->isEnqueued = true;
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
    $this->enqueue_scripts();
		//$this->enqueue_scripts();
	}

  function enqueue_scripts() {
    // Pro CSS
    $physical_file = MWL_PATH . '/app/style-pro.min.css';
    $cache_buster = file_exists( $physical_file ) ? filemtime( $physical_file ) : MWL_VERSION;
    wp_enqueue_style( 'style-pro-css', plugins_url( '/app/style-pro.min.css', __DIR__ ), null, $cache_buster );

    // Pro JS
    $physical_file = MWL_PATH . '/app/lightbox.js';
    $cache_buster = file_exists( $physical_file ) ? filemtime( $physical_file ) : MWL_VERSION;
    wp_enqueue_script( 'mwl-core-pro-js', plugins_url( '/app/lightbox-pro.js', __DIR__ ), null, $cache_buster, false );

    wp_localize_script('mwl-core-pro-js', 'mwl_map',
      array(
        'default_engine' => $this->core->get_option( 'map_engine', "" ),
        'googlemaps' => array(
          'api_key' => $this->core->get_option( 'googlemaps_token', '' ),
          'style' => json_decode( $this->core->get_option( 'googlemaps_style', '' ) )
        ),
        'mapbox' => array(
          'api_key' => $this->core->get_option( 'mapbox_token', '' ),
          'style' => json_decode( $this->core->get_option( 'mapbox_style', '' ) )
        ),
        'maptiler' => array(
          'api_key' => $this->core->get_option( 'maptiler_token', '' ),
          'style' => json_decode( $this->core->get_option( 'maptiler_style', '' ) )
        ),
        'zoom_level' => $this->core->get_option( 'map_zoom_level', 12 )
      )
    );
  }
}
