const TerserPlugin = require('terser-webpack-plugin');

module.exports = function (env, options) {
  return [
    // proConfig
    {
      mode: 'production',
      entry: './app/lightbox/pro.js',
      devtool: options.mode === 'production' ? false : 'source-map',
      optimization: {
        minimizer: [new TerserPlugin({
          extractComments: false,
        })],
      },
      output: {
        filename: 'lightbox-pro.js',
        chunkLoadingGlobal: 'wpJsonLightboxPro',
        path: __dirname + '/app/'
      },
      module: {
        rules: [
          { test: /\.js$/, exclude: /node_modules/, loader: "babel-loader" }
        ]
      }
    },
    // freeConfig
    {
      mode: 'production',
      entry: './app/lightbox/run.js',
      devtool: options.mode === 'production' ? false : 'source-map',
      optimization: {
        minimizer: [new TerserPlugin({
          extractComments: false,
        })],
      },
      output: {
        filename: 'lightbox.js',
        chunkLoadingGlobal: 'wpJsonLightbox',
        path: __dirname + '/app/'
      },
      module: {
        rules: [
          { test: /\.html$/i, loader: "html-loader" },
          { test: /\.js$/, exclude: /node_modules/, loader: "babel-loader" },
          {
            test: /\.js$/,
            exclude: /node_modules/,
            use: {
              loader: 'babel-loader',
              options: {
                presets: ["@babel/preset-env", "@babel/preset-react"],
                plugins: [
                  ["@babel/plugin-transform-react-jsx", {
                    runtime: "automatic",
                    importSource: "preact",
                    // pragma: "h",
                    // pragmaFrag: "Fragment",
                  }]
                ]
              }
            },
          }
        ]
      }
    }
  ];
};
